// commands, macros and stuff.

use crate::{errors::TexError, AsLatex, Res};

#[derive(Debug, Clone)]
pub struct Command {
    pub name: String,
    pub nargs: usize,
    pub def: String, // actual latex, cannot help
}

impl AsLatex for Command {
    fn to_string(&self) -> String {
        format!(
            "\\newcommand{{{}}}[{}]{{{}}} ",
            self.name, self.nargs, self.def
        )
    }
}

impl Command {
    pub fn new(name: String, nargs: usize, def: String) -> Self {
        Self { name, nargs, def }
    }

    pub fn call(&self, args: Vec<String>) -> Res<String> {
        if args.len() != self.nargs {
            return Err(TexError::ArgLen.into());
        }
        let mut q = args.iter();
        let temp = self
            .def
            .replace(r"#[0-9]+", "%%")
            .split("%")
            .map(|s| if s.is_empty() { q.next().unwrap() } else { s })
            .collect::<String>();
        Ok(temp)
    }
}
